/*Includes ----------------------------------------------*/
#include "tool.h"
#include "nrf_delay.h"
#include "system.h"
#include "ble_comm.h"
#include "bsp_time.h"
#include "hal_led.h"
#include "hal_charge.h"
#include "hal_ble_common.h"
#include "hal_ble_client.h"
#include "hal_ble_host.h"
#include "app_flash.h"
#include "app_wireless_pair.h"
#include "app_detectIsHost.h"

/*Private macro ------------------------------------------------------------------------------------------------------------------------------------*/
#define				APP_WIRELESS_PAIR_FSMTABLE_LEN															8																			//״̬													
#define				APP_WIRELESS_PAIR_RSSI_THRESHOLD														-18																		//rssiֵ
#define				APP_WIRELESS_PAIR_RSSI_BUFF_LEN															15																		//rssi

/*Struction ------------------------------------------------------------------------------------------------------------------------------------*/
//״̬
typedef struct _app_wireless_pair_fsmtable_s
{
	App_Wireless_Pair_Event_e												event;																												//¼
	App_Wireless_Pair_State_e												CurState;																											//ǰ״̬
	int																							(*eventActFun)();																							//ָ
	App_Wireless_Pair_State_e												NextState;																										//һ״̬
}App_Wireless_Pair_fsmtable_s;
//״̬
typedef struct _app_wireless_pair_fsm_s
{
	App_Wireless_Pair_State_e												curState;																											//ǰ״̬
	App_Wireless_Pair_fsmtable_s										FsmTable[APP_WIRELESS_PAIR_FSMTABLE_LEN];											//״̬
	App_Wireless_Pair_Set_Scan_Cb										set_scan_cb;																									//ɨص
	App_Wireless_Pair_Set_Adv_Cb										set_adv_cb;																										//ù㲥ص
	uint8_t																					peer_addr[6];																									//ԶMACַ
	bool																						is_busy;																											//Ƿæµ
	uint8_t																					led_display_count;																						//ƴ
	uint8_t																					led_switch;																										//ƵĿ
	char 																						already_scan_or_adv_mac_buf[16];															//Ѿɨ㲥macַ
	int 																						rssi_buf[APP_WIRELESS_PAIR_RSSI_BUFF_LEN];										//rssi
}App_Wireless_Pair_Fsm_s;
/*Local Variable ------------------------------------------------------------------------------------------------------------------------------------*/
static App_Wireless_Pair_Fsm_s ob_app_wireless_pair_fsm;

/*Local Functions ------------------------------------------------------------------------------------------------------------------------------------*/
static void app_wireless_pair_slow_clear_rssi_buff_process(void);

static int app_wireless_pair_set_scan_adv_on(void)
{
	int ret = 0;
	int adv_stop = -1;
	
	//Ьɨ״̬
	//ЬɨӦΪMACַ״̬
	if(mFlash.isHost)
	{
		if(slave_isconnect() == 0)
		{
			//״̬£ֹܵͣ㲥
			adv_stop = wireless_pair_advertising_stop();
			if(ob_app_wireless_pair_fsm.set_adv_cb != NULL && adv_stop == 0 && slave_isconnect() == 0)ret += ob_app_wireless_pair_fsm.set_adv_cb(0x01);
			else ret = -1;
		}
		nrf_ble_scan_stop();
		host_disconnect();
		//״̬£ֹܵͣɨ
		if(host_isconnect() == 0)
		{
			if(ob_app_wireless_pair_fsm.set_scan_cb != NULL)ret += ob_app_wireless_pair_fsm.set_scan_cb(0x00);
			else ret = -1;
		}
		else
		{
			ret = -1;
		}
	}
	else
	{
		if(host_isconnect() == 0)
		{
			//״̬£ֹܵͣɨ
			nrf_ble_scan_stop();
			if(ob_app_wireless_pair_fsm.set_scan_cb != NULL)ret += ob_app_wireless_pair_fsm.set_scan_cb(0x01);
			else ret = -1;
		}
		slave_disconnect();
		//״̬£ֹܵͣ㲥
		if(slave_isconnect() == 0)adv_stop = wireless_pair_advertising_stop();
		if(adv_stop == 0 && slave_isconnect() == 0)
		{
			if(ob_app_wireless_pair_fsm.set_adv_cb != NULL)ret += ob_app_wireless_pair_fsm.set_adv_cb(0x00);
			else ret = -1;
		}
		else
		{
			ret = -1;
		}
	}
	
	if(ret == 0)Process_Start(100,"app_wireless_pair_slow_clear_rssi_buff_process",app_wireless_pair_slow_clear_rssi_buff_process);
	
	return ret;
}
static int app_wireless_pair_set_scan_adv_off(void)
{
	int 			ret = 0;
	char 			buf[16];
	int 			adv_stop = -1;
	
	//Ь뱻ɨ״̬
	//ЬɨӦΪ״̬
	if(mFlash.isHost)
	{
		if(slave_isconnect() == 0)
		{
			//״̬£ֹܵͣ㲥
			adv_stop = wireless_pair_advertising_stop();
			if(ob_app_wireless_pair_fsm.set_adv_cb != NULL && adv_stop == 0 && slave_isconnect() == 0)ret += ob_app_wireless_pair_fsm.set_adv_cb(0x00);
			else ret = -1;
		}
		nrf_ble_scan_stop();
		host_disconnect();
		//״̬£ֹܵͣɨ
		if(host_isconnect() == 0)
		{
			if(ob_app_wireless_pair_fsm.set_scan_cb != NULL)
			{
				sprintf(buf,"%02X%02X%02X%02X%02X%02X",mFlash.macHost[0],mFlash.macHost[1],mFlash.macHost[2],mFlash.mClient.macAddr[3],mFlash.mClient.macAddr[4],mFlash.mClient.macAddr[5]);
				host_set_scan_name(buf,strlen(buf));
				ret += ob_app_wireless_pair_fsm.set_scan_cb(0x00);
			}
			else ret = -1;
		}
		else
		{
			ret = -1;
		}
	}
	else
	{
		if(host_isconnect() == 0)
		{
			//״̬£ֹܵͣɨ
			//ֹͣɨ裬ΪЬҪŴٶ
			nrf_ble_scan_stop();
		}
		slave_disconnect();
		//״̬£ֹܵͣ㲥
		if(slave_isconnect() == 0)adv_stop = wireless_pair_advertising_stop();
		if(adv_stop == 0 && slave_isconnect() == 0)
		{
			if(ob_app_wireless_pair_fsm.set_adv_cb != NULL)
			{
				if(mFlash.mClient.isConfig == 'C')
				{
					sprintf(buf,"%02X%02X%02X%02X%02X%02X",mFlash.mClient.macAddr[0],mFlash.mClient.macAddr[1],mFlash.mClient.macAddr[2],mFlash.macHost[3],mFlash.macHost[4],mFlash.macHost[5]);
				}
				else
				{
					ble_gap_addr_t m_addr;
					uint32_t err_code;
					//ȡmacַ
					err_code = sd_ble_gap_addr_get(&m_addr);
					APP_ERROR_CHECK(err_code);	
					for(int i=0;i<6;i++)mFlash.macHost[i] = m_addr.addr[5-i];
					sprintf(buf,"%02X%02X%02X%02X%02X%02X",mFlash.macHost[0],mFlash.macHost[1],mFlash.macHost[2],mFlash.macHost[3],mFlash.macHost[4],mFlash.macHost[5]);
				}
				slave_set_adv_name(buf,strlen(buf));
				slave_adv_init();
				ret += ob_app_wireless_pair_fsm.set_adv_cb(0x00);
			}
			else ret = -1;
		}
		else
		{
			ret = -1;
		}
	}
	
	if(ret == 0)
	{
		ob_app_wireless_pair_fsm.led_switch = 1;
		memset(&ob_app_wireless_pair_fsm.peer_addr,0xFF,sizeof(ob_app_wireless_pair_fsm.peer_addr));
		for(int i=0;i<APP_WIRELESS_PAIR_RSSI_BUFF_LEN;i++)ob_app_wireless_pair_fsm.rssi_buf[i] = -500;
		Process_Stop(app_wireless_pair_slow_clear_rssi_buff_process);
	}

	return ret;
}

static int app_wireless_pair_unconditional_entry(void)
{
	return 0;	//κβֱӽһ״̬
}

static int app_wireless_pair_enter_in_process(int rssi, uint8_t *addr)
{

	static uint8_t	index = 0;
	int 						ret = 0;
	int 						rssi_max = -500;
	uint8_t 				null_addr[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

	if(ob_app_wireless_pair_fsm.is_busy == false)
	{
		ob_app_wireless_pair_fsm.rssi_buf[index++] = rssi;
		if(index == APP_WIRELESS_PAIR_RSSI_BUFF_LEN)index = 0;
		
		for(int i=0;i<APP_WIRELESS_PAIR_RSSI_BUFF_LEN;i++)rssi_max = (rssi_max > ob_app_wireless_pair_fsm.rssi_buf[i])?rssi_max:ob_app_wireless_pair_fsm.rssi_buf[i];
		
		
		
		
		
//		SEGGER_RTT_printf(0,"rssi:%d,%d,st:%d,curState:%d,active:%d,host:%d,slave:%d,mFlash.isHost:%d,led:%d,mac: ",rssi_max,rssi,NRF_RTC0->COUNTER, \
//																																										 ob_app_wireless_pair_fsm.curState,wireless_pair_scan_get_active(),\
//		                                                                                 host_isconnect(),slave_isconnect(),mFlash.isHost,ob_app_wireless_pair_fsm.led_switch);
		
		if(mFlash.isHost)
		{
			extern char Target_scan[TARFET_LEN_MAX];
//			for(int i=0; i<6; i++)SEGGER_RTT_printf(0,"0x%x ",Target_scan[5-i]);
		}
		else 
		{
			int len;
			char name[16];
			slave_get_advname_len(&len);
			slave_get_advname(name,len);
//			for(int i=0; i<6; i++)SEGGER_RTT_printf(0,"0x%x ",name[5-i]);
		}
		
//		if(rssi >= APP_WIRELESS_PAIR_RSSI_THRESHOLD)
//		{
//			SEGGER_RTT_printf(0,"  rssi_mac : ");for(int i=0; i<6; i++)SEGGER_RTT_printf(0,"0x%x ",addr[5-i]);SEGGER_RTT_printf(0,"\r\n");
//		}
//		else
//			SEGGER_RTT_printf(0,"\r\n");
		
		
		if(memcmp(addr,null_addr,6) != 0 && rssi >=APP_WIRELESS_PAIR_RSSI_THRESHOLD)
		{
			for(int i=0; i<6; i++)ob_app_wireless_pair_fsm.peer_addr[i] = addr[5-i];//ȷrssiӦmac
			ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
		}
		else if(rssi_max < APP_WIRELESS_PAIR_RSSI_THRESHOLD)
		{
			if(mFlash.isHost && host_isconnect() == 1)
			{
				ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
			}
			else if(!mFlash.isHost && slave_isconnect() == 1)
			{
				ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
			}
			else ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_DOES_NOT_MEETS_REQUIREMENTS);
		}
		else
		{
			if(memcmp(addr,null_addr,6) != 0)
			{
				ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
			}
			else
			{
				if(mFlash.isHost && host_isconnect() == 1)
				{
					ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
				}
				else if(!mFlash.isHost && slave_isconnect() == 1)
				{
					ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
				}
				else ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_DOES_NOT_MEETS_REQUIREMENTS);
			}
		}
		
		
		
		
//		if(rssi_max >= APP_WIRELESS_PAIR_RSSI_THRESHOLD)
//		{
//			if(memcmp(addr,null_addr,6) != 0)
//			{
//				ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
//			}
//			else
//			{
//				if(!mFlash.isHost && slave_isconnect() == 1)
//				{
//					ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
//				}
//				else ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_DOES_NOT_MEETS_REQUIREMENTS);
//			}
//		}
//		else
//		{
//			if(mFlash.isHost && host_isconnect() == 1)
//			{
//				ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
//			}
//			else if(!mFlash.isHost && slave_isconnect() == 1)
//			{
//				ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS);
//			}
//			else ret = app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_DOES_NOT_MEETS_REQUIREMENTS);
//		}
	}
	
	return ret;
}

static void app_wireless_pair_slow_clear_rssi_buff_process(void)
{
	uint8_t addr[6];
	memset(addr,0x00,6);
	app_wireless_pair_enter_in_process(APP_WIRELESS_PAIR_RSSI_THRESHOLD*2,addr);
}

static void display_led_process(void)
{
	ob_app_wireless_pair_fsm.led_display_count++;
	if (ob_app_wireless_pair_fsm.led_display_count % 2 == 0)
	{
			LED_Start(LED_PAIR, COLOR_BLACK);
	}
	else
	{
			LED_Start(LED_PAIR, COLOR_BLUE);
	}
	if (ob_app_wireless_pair_fsm.led_display_count > 20)
	{
			ob_app_wireless_pair_fsm.led_display_count = 0;
			LED_Stop(LED_PAIR);
			Process_SetHoldOn(display_led_process, 0);
			Process_Stop(display_led_process);
	}
}

static void app_wireless_pair_start_display_led(void)
{
	ob_app_wireless_pair_fsm.led_display_count = 0;
  Process_Start(500, "display_led_process", display_led_process);
  Process_SetHoldOn(display_led_process, 1);
}

static void app_wireless_pair_stop_display_led(void)
{
	ob_app_wireless_pair_fsm.led_display_count = 0;
	LED_Stop(LED_PAIR);
	Process_SetHoldOn(display_led_process, 0);
	Process_Stop(display_led_process);
}

static void app_wireless_pair_in_process(void)
{
	int	ret = 0;
	uint8_t *connect_addr;
	uint8_t peer_addr[6];																																				//Զmacַ
	ble_gap_addr_t m_addr;																																			//macַ
	char buf[16];
	uint32_t err_code;
	int adv_stop = -1;
	uint8_t null_addr[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

	if(!get_LR_readly())
	{
		return;
	}
	
	if(ob_app_wireless_pair_fsm.curState == APP_WIRELESS_PAIR_STATE_PAIRING_IN_PROGRESS)
	{
		ob_app_wireless_pair_fsm.is_busy = true;
		
		//ȡӵĶԶmacַ
		connect_addr = wireless_pair_get_connect_macaddr();
		//ȡmacַ
		err_code = sd_ble_gap_addr_get(&m_addr);
		APP_ERROR_CHECK(err_code);
		//ȡԶ˵macַ
		for(int i=0;i<6;i++)peer_addr[i]=ob_app_wireless_pair_fsm.peer_addr[i];
		
		
		if(mFlash.isHost)																			//Ь߼
		{
			//жǷӵmacַΪԵַ
			if(host_isconnect() == 1 && memcmp(peer_addr,connect_addr,6) == 0)
			{
				//жǷԹԹֱӽԽ׶ΡԹ¼MACַԽ
				if(memcmp(peer_addr,mFlash.mClient.macAddr,6) != 0)
				{
					//Ь㲥֣״̬£ֹܵͣ㲥
					if(slave_isconnect() == 0)adv_stop = wireless_pair_advertising_stop();
					if(adv_stop == 0 && slave_isconnect() == 0)
					{
						sprintf(buf,"SH_%02X%02X",m_addr.addr[4], m_addr.addr[5]);
						slave_set_adv_name(buf,strlen(buf));
						slave_adv_init();
						ret = ob_app_wireless_pair_fsm.set_adv_cb(0x01);
						if(ret == 0)
						{
							for(int i = 0; i < 6; i++)
							{
								mFlash.mClient.macAddr[i] = peer_addr[i];
								mFlash.macHost[i] 				= m_addr.addr[5-i];
								mBackup.macAddr_L[i] 			= mFlash.macHost[i];         //ַ
								mBackup.macAddr_R[i] 			= mFlash.mClient.macAddr[i]; //ӻַ
							}
							mFlash.mClient.isConfig 		='C';
							mBackup.hardVersion 				= mFlash.mClient.hardVersion;
							mBackup.sotfVersion 				= mFlash.mClient.sotfVersion;
							mBackup.isConfig 						= mFlash.mClient.isConfig;
							if(Flash_SaveBackup() != ZONE_OP_SUCCESS){Except_TxError(EXCEPT_Pair, "pair save backup fail");}
							if(Flash_DeleteAllStep() != ZONE_OP_SUCCESS){Except_TxError(EXCEPT_Pair, "pair clear step fail");}
							memset(&mFlash.mStep, 0, sizeof(FlashStep_t));
							if(Flash_SaveInfomation() != ZONE_OP_SUCCESS){Except_TxError(EXCEPT_Pair, "Flash_SaveInfomation fail");}
							app_wireless_pair_start_led();//ûҪӲŻῪ̡߳
							app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_PAIR_DONE);
						}
					}
				}
				else
				{
					app_wireless_pair_start_led();//ûҪӲŻῪ̡߳
					app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_PAIR_DONE);
				}
				
				if(ob_app_wireless_pair_fsm.curState == APP_WIRELESS_PAIR_STATE_END_OF_PAIRING && host_get_rssi() >= APP_WIRELESS_PAIR_RSSI_THRESHOLD)
				{
					if(slave_isconnect() == 0)adv_stop = wireless_pair_advertising_stop();
					if(adv_stop == 0 && slave_isconnect() == 0)
					{
						ob_app_wireless_pair_fsm.set_adv_cb(0x00);
					}
				}
			}
			else
			{
				//ǣϿӣɨΪµԵַ
				sprintf(buf,"%02X%02X%02X%02X%02X%02X",m_addr.addr[5],m_addr.addr[4],m_addr.addr[3],peer_addr[3],peer_addr[4],peer_addr[5]);
				if(memcmp(ob_app_wireless_pair_fsm.already_scan_or_adv_mac_buf,buf,strlen(buf)) != 0 && memcmp(peer_addr,null_addr,6) != 0)
				{
//					SEGGER_RTT_printf(0,"scan_or_adv_mac_buf:  %s\r\n",ob_app_wireless_pair_fsm.already_scan_or_adv_mac_buf);
//					SEGGER_RTT_printf(0,"buf: %s\r\n",buf);
					if(slave_isconnect() == 0)
					{
						//״̬£ֹܵͣ㲥
						adv_stop = wireless_pair_advertising_stop();
						if(adv_stop == 0 && slave_isconnect() == 0)ret += ob_app_wireless_pair_fsm.set_adv_cb(0x01);
					}
					nrf_ble_scan_stop();
					host_disconnect();
					//״̬£ֹܵͣɨ
					if(host_isconnect() == 0)
					{
						host_set_scan_name(buf,strlen(buf));
						ret += ob_app_wireless_pair_fsm.set_scan_cb(0x02);
						if(ret == 0){memcpy(ob_app_wireless_pair_fsm.already_scan_or_adv_mac_buf,buf,strlen(buf));}
					}
				}
			}
		}
		else																									//Ь߼
		{
			//жǷӵmacַΪԵַ
			if(slave_isconnect() == 1 && memcmp(peer_addr,connect_addr,6) == 0)
			{
				//жǷԹԹֱӽԽ׶ΡԹ¼MACַԽ
				if(memcmp(peer_addr,mFlash.mClient.macAddr,6) != 0)
				{
					for(int i = 0; i < 6; i++)
					{
						mFlash.mClient.macAddr[i]	=	peer_addr[i];
						mFlash.macHost[i]					=	m_addr.addr[5-i];
						mBackup.macAddr_L[i] 			= mFlash.macHost[i];         //ַ
						mBackup.macAddr_R[i] 			= mFlash.mClient.macAddr[i]; //ӻַ
					}
					mFlash.mClient.isConfig 		='C';
					mBackup.hardVersion 				= mFlash.mClient.hardVersion;
					mBackup.sotfVersion 				= mFlash.mClient.sotfVersion;
					mBackup.isConfig 						= mFlash.mClient.isConfig;
					if(Flash_SaveBackup() != ZONE_OP_SUCCESS){Except_TxError(EXCEPT_Pair, "pair save backup fail");}
					if(Flash_DeleteAllStep() != ZONE_OP_SUCCESS){Except_TxError(EXCEPT_Pair, "pair clear step fail");}
					memset(&mFlash.mStep, 0, sizeof(FlashStep_t));
					if(Flash_SaveInfomation() != ZONE_OP_SUCCESS){Except_TxError(EXCEPT_Pair, "Flash_SaveInfomation fail");}
				}
				app_wireless_pair_start_led();//ûҪӲŻῪ̡߳
				app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_PAIR_DONE);
				if(ob_app_wireless_pair_fsm.curState == APP_WIRELESS_PAIR_STATE_END_OF_PAIRING  && slave_get_rssi() >= APP_WIRELESS_PAIR_RSSI_THRESHOLD)
				{
					if(host_isconnect() == 0)nrf_ble_scan_stop();
				}
			}
			else
			{
				//ǣϿӣù㲥ΪµԵַ
				sprintf(buf,"%02X%02X%02X%02X%02X%02X",peer_addr[0],peer_addr[1],peer_addr[2],m_addr.addr[2],m_addr.addr[1],m_addr.addr[0]);
				if(memcmp(ob_app_wireless_pair_fsm.already_scan_or_adv_mac_buf,buf,strlen(buf)) != 0 && memcmp(peer_addr,null_addr,6) != 0)
				{
					if(host_isconnect() == 0)
					{
						//״̬£ֹܵͣɨ
						nrf_ble_scan_stop();
						ret += ob_app_wireless_pair_fsm.set_scan_cb(0x01);
					}
					slave_disconnect();
					//״̬£ֹܵͣ㲥
					if(slave_isconnect() == 0)adv_stop = wireless_pair_advertising_stop();
					if(adv_stop == 0 && slave_isconnect() == 0)
					{
						DEBUG_LOG("pair : advName(%d):%s\n",strlen(buf),buf);
						slave_set_adv_name(buf,strlen(buf));
						slave_adv_init();
						ret += ob_app_wireless_pair_fsm.set_adv_cb(0x02);
						if(ret == 0){memcpy(ob_app_wireless_pair_fsm.already_scan_or_adv_mac_buf,buf,strlen(buf));}
					}
				}
			}
		}
		ob_app_wireless_pair_fsm.is_busy = false;
	}
}

/*API ------------------------------------------------------------------------------------------------------------------------------------*/
/**
	@brief 		ʼӦ
	@param 		
	@return 						- [out]	-1ʧܣ0ɹ
*/
int	app_wireless_pair_Init(void)
{
	//ʼṹ
//	memset(&ob_app_wireless_pair_fsm,0,sizeof(App_Wireless_Pair_Fsm_s));																													//mesetʼΪصᱻ
	
	ob_app_wireless_pair_fsm.curState 								= APP_WIRELESS_PAIR_STATE_PAIRING_STANDBY;
	
	ob_app_wireless_pair_fsm.is_busy 									= false;
	
	ob_app_wireless_pair_fsm.led_switch								= 1;
	
	memset(&ob_app_wireless_pair_fsm.peer_addr,0xFF,sizeof(ob_app_wireless_pair_fsm.peer_addr));
	for(int i=0;i<APP_WIRELESS_PAIR_RSSI_BUFF_LEN;i++)ob_app_wireless_pair_fsm.rssi_buf[i] = -500;
	
	//{¼ǰ״̬ҪҪִеĺһ״̬}
	ob_app_wireless_pair_fsm.FsmTable[0].event				=	APP_WIRELESS_PAIR_EVENT_CHARGING_INSERTING;																	//
	ob_app_wireless_pair_fsm.FsmTable[0].CurState			=	APP_WIRELESS_PAIR_STATE_PAIRING_STANDBY;																		//Դ״̬
	ob_app_wireless_pair_fsm.FsmTable[0].eventActFun	=	app_wireless_pair_set_scan_adv_on;																					//ɨ͹㲥
	ob_app_wireless_pair_fsm.FsmTable[0].NextState		=	APP_WIRELESS_PAIR_STATE_PAIRING_PREPARATION;																//׼״̬
	
	ob_app_wireless_pair_fsm.FsmTable[1].event				=	APP_WIRELESS_PAIR_EVENT_CHARGING_UNPLUG;																		//γ
	ob_app_wireless_pair_fsm.FsmTable[1].CurState			=	APP_WIRELESS_PAIR_STATE_PAIRING_PREPARATION;																//׼״̬
	ob_app_wireless_pair_fsm.FsmTable[1].eventActFun	=	app_wireless_pair_set_scan_adv_off;																					//ɨ͹㲥
	ob_app_wireless_pair_fsm.FsmTable[1].NextState		=	APP_WIRELESS_PAIR_STATE_PAIRING_STANDBY;																		//Դ״̬
	
	ob_app_wireless_pair_fsm.FsmTable[2].event				=	APP_WIRELESS_PAIR_EVENT_RSSI_MEETS_REQUIREMENTS;														//rssiҪ
	ob_app_wireless_pair_fsm.FsmTable[2].CurState			=	APP_WIRELESS_PAIR_STATE_PAIRING_PREPARATION;																//׼״̬
	ob_app_wireless_pair_fsm.FsmTable[2].eventActFun	=	app_wireless_pair_unconditional_entry;																			//ֱӽһ״̬
	ob_app_wireless_pair_fsm.FsmTable[2].NextState		=	APP_WIRELESS_PAIR_STATE_PAIRING_IN_PROGRESS;																//Խ״̬
	
	ob_app_wireless_pair_fsm.FsmTable[3].event				=	APP_WIRELESS_PAIR_EVENT_CHARGING_UNPLUG;																		//γ
	ob_app_wireless_pair_fsm.FsmTable[3].CurState			=	APP_WIRELESS_PAIR_STATE_PAIRING_IN_PROGRESS;																//Խ״̬
	ob_app_wireless_pair_fsm.FsmTable[3].eventActFun	=	app_wireless_pair_set_scan_adv_off;																					//ɨ͹㲥
	ob_app_wireless_pair_fsm.FsmTable[3].NextState		=	APP_WIRELESS_PAIR_STATE_PAIRING_STANDBY;																		//Դ״̬
	
	ob_app_wireless_pair_fsm.FsmTable[4].event				=	APP_WIRELESS_PAIR_EVENT_RSSI_DOES_NOT_MEETS_REQUIREMENTS;										//rssiҪ
	ob_app_wireless_pair_fsm.FsmTable[4].CurState			=	APP_WIRELESS_PAIR_STATE_PAIRING_IN_PROGRESS;																//Խ״̬
	ob_app_wireless_pair_fsm.FsmTable[4].eventActFun	=	app_wireless_pair_unconditional_entry;																			//ֱӽһ״̬
	ob_app_wireless_pair_fsm.FsmTable[4].NextState		=	APP_WIRELESS_PAIR_STATE_PAIRING_PREPARATION;																//׼״̬
	
	ob_app_wireless_pair_fsm.FsmTable[5].event				=	APP_WIRELESS_PAIR_EVENT_PAIR_DONE;																					//					
	ob_app_wireless_pair_fsm.FsmTable[5].CurState			=	APP_WIRELESS_PAIR_STATE_PAIRING_IN_PROGRESS;																//Խ״̬
	ob_app_wireless_pair_fsm.FsmTable[5].eventActFun	=	app_wireless_pair_unconditional_entry;																			//ֱӽһ״̬
	ob_app_wireless_pair_fsm.FsmTable[5].NextState		=	APP_WIRELESS_PAIR_STATE_END_OF_PAIRING;																			//Խ״̬
	
	ob_app_wireless_pair_fsm.FsmTable[6].event				=	APP_WIRELESS_PAIR_EVENT_CHARGING_UNPLUG;																		//γ	
	ob_app_wireless_pair_fsm.FsmTable[6].CurState			=	APP_WIRELESS_PAIR_STATE_END_OF_PAIRING;																			//Խ״̬
	ob_app_wireless_pair_fsm.FsmTable[6].eventActFun	=	app_wireless_pair_set_scan_adv_off;																					//ɨ͹㲥
	ob_app_wireless_pair_fsm.FsmTable[6].NextState		=	APP_WIRELESS_PAIR_STATE_PAIRING_STANDBY;																		//Դ״̬
	
	ob_app_wireless_pair_fsm.FsmTable[7].event				=	APP_WIRELESS_PAIR_EVENT_RSSI_DOES_NOT_MEETS_REQUIREMENTS;										//rssiҪ
	ob_app_wireless_pair_fsm.FsmTable[7].CurState			=	APP_WIRELESS_PAIR_STATE_END_OF_PAIRING;																			//Խ״̬
	ob_app_wireless_pair_fsm.FsmTable[7].eventActFun	=	app_wireless_pair_unconditional_entry;																			//ֱӽһ״̬
	ob_app_wireless_pair_fsm.FsmTable[7].NextState		=	APP_WIRELESS_PAIR_STATE_PAIRING_PREPARATION;																//׼״̬
	
	Process_Start(0,"app_wireless_pair_in_process",app_wireless_pair_in_process);
	
	return 0;
}

/**
	@brief 		עɨ
	@param 		cb								-	[in]	עĻص
	@return 						- [out]	-1ʧܣ0ɹ
*/
int	app_wireless_pair_Regist_SetScan(App_Wireless_Pair_Set_Scan_Cb cb)
{
	if(cb != NULL)
	{
		ob_app_wireless_pair_fsm.set_scan_cb = cb;
		return 0;
	}
	return -1;
}

/**
	@brief 		ע㲥
	@param 		cb								-	[in]	עĻص
	@return 						- [out]	-1ʧܣ0ɹ
*/
int	app_wireless_pair_Regist_SetAdv(App_Wireless_Pair_Set_Adv_Cb cb)
{
	if(cb != NULL)
	{
		ob_app_wireless_pair_fsm.set_adv_cb = cb;
		return 0;
	}
	return -1;
}

/**
	@brief 		ɨ
	@param 		rssi							-	[in]	źǿ
	@param 		addr							-	[in]	MACַָ
	@return 						- [out]	-1ʧܣ0ɹ
*/
int	app_wireless_pair_Scan_Req(int rssi, uint8_t *addr)
{
	int ret = 0;
	
	if(mFlash.isHost)
	{
		app_wireless_pair_enter_in_process(rssi,addr);
	}
	else
	{
		ret = -1;					//ֻЬյЬɨŶ
	}
	
	return ret;
}

/**
	@brief 		ɨظ
	@param 		rssi							-	[in]	źǿ
	@param 		addr							-	[in]	MACַָ
	@return 						- [out]	-1ʧܣ0ɹ
*/
int	app_wireless_pair_Scan_Rsp(int rssi, uint8_t *addr)
{
	int ret = 0;
	
	if(!mFlash.isHost)
	{
		app_wireless_pair_enter_in_process(rssi,addr);
	}
	else
	{
		ret = -1;					//ֻЬյЬɨӦŶ
	}
	
	return ret;
}

/**
	@brief 		
	@param 		
	@return 						- [out]	-1ʧܣ0ɹ
*/
int	app_wireless_pair_start_led(void)
{
	if(ob_app_wireless_pair_fsm.led_switch != 0 && (ob_app_wireless_pair_fsm.curState == APP_WIRELESS_PAIR_STATE_PAIRING_IN_PROGRESS))
	{
		ob_app_wireless_pair_fsm.led_switch--;
		app_wireless_pair_start_display_led();
//		SEGGER_RTT_printf(0,"app_wireless_pair_start_display_led\r\n");
	}
	return 0;
}

/**
	@brief 		
	@param 		
	@return 						- [out]	-1ʧܣ0ɹ
*/
int	app_wireless_pair_stop_led(void)
{
	app_wireless_pair_stop_display_led();
	app_wireless_pair_EventHandle(APP_WIRELESS_PAIR_EVENT_RSSI_DOES_NOT_MEETS_REQUIREMENTS);
//	SEGGER_RTT_printf(0,"app_wireless_pair_stop_led\r\n");
	return 0;
}

/**
	@brief 		¼
	@param 		event							-	[in]	¼
	@return 						- [out]	-1ʧܣ0ɹ
*/
int	app_wireless_pair_EventHandle(App_Wireless_Pair_Event_e  event)
{
	int (*eventActFun)() = NULL;  	//ָʼΪ
	App_Wireless_Pair_State_e NextState;
	int flag = 0; 									//ʶǷ
	int i;
	int ret = 0;
	
	
	/*ȡǰ*/
	for (i = 0; i<APP_WIRELESS_PAIR_FSMTABLE_LEN; i++)
	{
			//ҽǰ״ָ̬¼Ҳִ
			if (event == ob_app_wireless_pair_fsm.FsmTable[i].event && ob_app_wireless_pair_fsm.curState == ob_app_wireless_pair_fsm.FsmTable[i].CurState)
			{
					flag = 1;
					eventActFun = ob_app_wireless_pair_fsm.FsmTable[i].eventActFun;
					NextState = ob_app_wireless_pair_fsm.FsmTable[i].NextState;
					break;
			}
	}
	
	if (flag) //
	{
		/*ִ*/
		if(eventActFun)
		{
			ret = eventActFun();
		}

		//תһ״̬
		if(ret == 0)
		{
			ob_app_wireless_pair_fsm.curState = NextState;
//			SEGGER_RTT_printf(0,"success state:%d, event:%d\r\n",ob_app_wireless_pair_fsm.curState,event);
		}
		else
		{
//			SEGGER_RTT_printf(0,"*****************************************fail state:%d, event:%d\r\n",ob_app_wireless_pair_fsm.curState,event);
		}
	}
	else
	{
		//there is no match
		ret = -1;
	}
	
	return ret;
}

App_Wireless_Pair_State_e app_wireless_pair_get_curstate(void)
{
	return ob_app_wireless_pair_fsm.curState;
}















