#ifndef __QUEUE_H
#define __QUEUE_H

/**@brief   ģʽ
 * @details 
 *          Also, it keeps the information about which memory is allocated for the buffer
 *          and its size. This structure must be initialized by app_fifo_init() before use.
 */
typedef enum
{
    QUEUE_MODE_OVERFLOW,        //!< If the queue is full, new element will overwrite the oldest.
    QUEUE_MODE_NO_OVERFLOW,     //!< If the queue is full, new element will not be accepted.
} queue_mode_t;
/**@brief   A FIFO instance structure.
 * @details Keeps track of which bytes to read and write next.
 *          Also, it keeps the information about which memory is allocated for the buffer
 *          and its size. This structure must be initialized by app_fifo_init() before use.
 */
typedef enum
{
	size_4 		  = 0x0004U,
	size_8 		  = 0x0008U,
	size_16 		= 0x0010U,								 	                            
	size_32 		= 0x0020U,
	size_64 		= 0x0040U,
	size_128 		= 0x0080U,
  size_256 		= 0x0100U,         
	size_512 		= 0x0200U,
	size_1024 	= 0x0400U,
	size_2048 	= 0x0800U,
	size_4096 	= 0x1000U,
	size_8192 	= 0x2000U,
	size_16384	= 0x4000U,
	size_32768	= 0x8000U,
} queue_size_t;

typedef struct
{
  volatile unsigned short front;          //!< Queue front index.
  volatile unsigned short back;           //!< Queue back index.
	void           		* p_buffer;          //!< Pointer to the memory that is used as storage.
	queue_size_t  		size;              //!< Size of the queue.
	unsigned short    element_count;
	unsigned short  	element_size;      //!< Size of one element.
	queue_mode_t      mode;
} queue_t;

/******************************API ӿ*************************************/
#define QUEUE_DEF(element_type, _name, _size, _mode)        \
				element_type   _name##_queue_buffer[(_size)];      	\
				queue_t _name=                             					\
								{                             							\
									.front 					= 0,              				\
									.back 					= 0,                    	\
									.p_buffer 			= _name##_queue_buffer,   \
									.size  					= (_size),          			\
									.element_count	= 0,											\
									.element_size   = sizeof(element_type),   \
									.mode           = _mode,       						\
								}    

//Ԫؽ
unsigned short queue_in(queue_t * p_queue, void const * p_element);
//Ԫس
unsigned short queue_out(queue_t * p_queue, void * p_element);
//鿴һԪأԪزûм
unsigned short queue_peek(queue_t * p_queue, void * p_element);
//ն
void queue_reset(queue_t * p_queue);	
//ȡԪظ
unsigned short queue_element_count(queue_t * p_queue);	
//鿴ָ±ԪصĴеԪ±index==0ʼ								
unsigned short queue_read(queue_t * p_queue, void * p_element ,int index);		
//ӵһԪ								
unsigned short queue_throw_element(queue_t * p_queue);
#endif 
