/*********************************************************************
 * INCLUDES
 */
#include "sdk_common.h"
#include "SEGGER_RTT.h"
#include "usr_config.h"
#include "exception.h"
#include "system.h"
#include "nrf_gpio.h"
#include "bsp_pwm.h"

/*********************************************************************
 * DEFINITIONS
 */
#define PWM_INSTANCE						0

/*********************************************************************
 * LOCAL VARIABLES
 */
static nrf_drv_pwm_t m_pwm = NRF_DRV_PWM_INSTANCE(PWM_INSTANCE);

static nrf_drv_pwm_config_t m_config =
{
		.output_pins =
		{
				NRF_DRV_PWM_PIN_NOT_USED, // channel 0
				NRF_DRV_PWM_PIN_NOT_USED, // channel 1
				NRF_DRV_PWM_PIN_NOT_USED, // channel 2
				NRF_DRV_PWM_PIN_NOT_USED	 // channel 3
		},
		.irq_priority = PWM0_IRQ_PRIORITY,
		.base_clock   = NRF_PWM_CLK_125kHz,
		.count_mode   = NRF_PWM_MODE_UP,
		.top_value    = 15612,
		.load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
		.step_mode    = NRF_PWM_STEP_AUTO
};

static nrfx_pwm_handler_t m_pwm_callback = NULL;

static void bsp_pwm_init_process(void)
{
	if(Except_TxError(EXCEPT_PWM_INIT,"bsp_pwm_init_error\r\n") == 0)
	{
		Process_Stop(bsp_pwm_init_process);
	}
}

/*********************************************************************
 * LOCAL FUNCTIONS
 */
/**
	@brief PWM ʼ
	@prama 
	@return 
*/
static void Pwm_init(void)
{
	int 				ret = 0;
	ret_code_t 	errCode;
	
	errCode = nrf_drv_pwm_init(&m_pwm, &m_config, m_pwm_callback);
	if(errCode != NRF_SUCCESS)ret = -1;
	
	if(ret == -1)
	{
		Process_Start(0,"pwm_init",bsp_pwm_init_process);
	}
}
///**
//	@brief PWM Ѳ
//	@prama t - [in] ʱ
//	@return 
//*/
//static void cb_pwmWakeup(uint32_t t)
//{
//	Pwm_init();
//}
///**
//	@brief PWM ˯߲
//	@prama t - [in] ˯ʱ
//	@return 
//*/
//static void cb_pwmSleep(uint32_t t)
//{
//	nrfx_pwm_uninit(&m_pwm);
//}
/**
 @brief PWMͨļģʽ
 @param load_mode -[in] ģʽ
 @return 
*/
static void SetPwm_ChannelsLoadMode(nrf_pwm_dec_load_t load_mode)
{
	m_config.load_mode = load_mode;
}
/*********************************************************************
 * PUBLIC FUNCTIONS
 */
/**
 @brief PWMͨ
 @param p_pins -[in] ͨ
 @return 
*/
void SetPwm_Channels(uint32_t channel_1, uint32_t channel_2, uint32_t channel_3, uint32_t channel_4)
{
	m_config.output_pins[0] = channel_1;
	m_config.output_pins[1] = channel_2;
	m_config.output_pins[2] = channel_3;
	m_config.output_pins[3] = channel_4;
}
/**
 @brief PWMжϻص
 @param pwm_callback -[in] жϺĵַ
 @param irq_priority -[in] жϺȼ
 @return 
*/
void SetPwm_IRQ(nrfx_pwm_handler_t pwm_callback, uint8_t irq_priority)
{
	m_pwm_callback = pwm_callback;
	m_config.irq_priority = irq_priority;
}
/**
 @brief PWMĻʱ
 @param clock -[in] ʱƵ
 @return 
*/
void SetPwm_BaseClock(nrf_pwm_clk_t clock)
{
	m_config.base_clock = clock;
}
/**
 @brief PWMļģʽ
 @param count_mode -[in] ģʽ
 @return 
*/
void SetPwm_CountMode(nrf_pwm_mode_t count_mode)
{
	m_config.count_mode = count_mode;
}
/**
 @brief PWMռձֵ
 @param Max_duty_cycle_value -[in] ռձֵƵʾ
 @return 
*/
void SetPwm_DutyCycleThreshold(uint16_t Max_duty_cycle_value)
{
	m_config.top_value = Max_duty_cycle_value;
}
/**
 @brief PWMеƽģʽ,һcycleĽзʽ
 @param step_mode -[in] ƽģʽ
 @return 
*/
void SetPwm_SequenceStepMode(nrf_pwm_dec_step_t step_mode)
{
	m_config.step_mode = step_mode;
}
/**
 @brief PWM ĳʼ
 @param 
 @return 
*/
void Pwm_Initialize(void){
	Pwm_init();
}
/**
 @brief PWM δʼ
 @param 
 @return 
*/
void Pwm_UnInitialize(void)
{
	nrfx_pwm_uninit(&m_pwm);
}



/**
 @brief һͨ
 @param p_seqValues -[in]	ָһͨ
 @param	seqValues_length - [in]	еĴС
 @param seqValues_repeats -[in] еÿռձظĴ
 @param	seqValues_end_delay - [in]  ÿУһռձظĴ
 @return 
*/
nrf_pwm_sequence_t* Pwm_SetIndSequence(pwm_values_individual_t *p_seqValues, uint16_t seqValues_length,  uint32_t seqValues_repeats, uint32_t seqValues_end_delay)
{
	static nrf_pwm_sequence_t m_sequence;
	
	if(m_config.load_mode != NRF_PWM_LOAD_INDIVIDUAL)
	{
		SetPwm_ChannelsLoadMode(NRF_PWM_LOAD_INDIVIDUAL);
		Pwm_UnInitialize();
		Pwm_Initialize();
	}
	m_sequence.values.p_individual = (nrf_pwm_values_individual_t*)p_seqValues;
	m_sequence.length = seqValues_length;
	m_sequence.repeats = seqValues_repeats;
	m_sequence.end_delay = seqValues_end_delay;
	
	return &m_sequence;
}
/**
 @brief һͨ
 @param p_seqValues -[in]	ָһͨ
 @param	seqValues_length - [in]	еĴС
 @param seqValues_repeats -[in] еÿռձظĴ
 @param	seqValues_end_delay - [in]  ÿУһռձظĴ
 @return 
*/
nrf_pwm_sequence_t* Pwm_SetComSequence(pwm_values_common_t *p_seqValues, uint16_t seqValues_length,  uint32_t seqValues_repeats, uint32_t seqValues_end_delay)
{
	static nrf_pwm_sequence_t m_sequence;
	
	if(m_config.load_mode != NRF_PWM_LOAD_COMMON)
	{
		SetPwm_ChannelsLoadMode(NRF_PWM_LOAD_COMMON);
		Pwm_UnInitialize();
		Pwm_Initialize();
	}
	m_sequence.values.p_common = (nrf_pwm_values_common_t*)p_seqValues;
	m_sequence.length = seqValues_length;
	m_sequence.repeats = seqValues_repeats;
	m_sequence.end_delay = seqValues_end_delay;
	
	return &m_sequence;
}
/**
 @brief PWMָеĲ
 @param pwm_sequence -[in] еֵ
 @param	playback_count -[in] Ŵ
 @param	flags - [in]	ģʽ
 @return 
*/
uint32_t SetSimplePwmPlayBack(nrf_pwm_sequence_t *pwm_sequence, uint16_t playback_count, uint32_t flags)
{
	return nrf_drv_pwm_simple_playback(&m_pwm, pwm_sequence, playback_count, flags);
}
/**
 @brief PWMָеĲ
 @param pwm_sequence0 -[in] еֵ
 @param pwm_sequence1 -[in] еֵ
 @param	playback_count -[in] Ŵ
 @param	flags - [in]	ģʽ
 @return 
*/
uint32_t SetComplexPwmPlayBack(nrf_pwm_sequence_t *pwm_sequence0, nrf_pwm_sequence_t *pwm_sequence1, uint16_t playback_count, uint32_t flags)
{
	return nrf_drv_pwm_complex_playback(&m_pwm, pwm_sequence0, pwm_sequence1, playback_count, flags);
}

